import './config.dart';

class Message {
  static generate(String name, List<Map<String, dynamic>> payloads) {
    switch (name) {
      case "join":
        return join(payloads);
        break;
      case "publish":
        return publish(payloads);
        break;
      case "subscribe":
        return subscribe(payloads);
        break;
      case "unpublish":
        return unpublish(payloads);
        break;
      case "unsubscribe":
        return unsubscribe(payloads);
        break;
      case "publishStats":
        return publishStats(payloads);
        break;
      case "subscribeStats":
        return subscribeStats(payloads);
        break;
      case "muteVideo":
        return muteVideo(payloads);
        break;
      case "muteAudio":
        return muteAudio(payloads);
        break;
      case "unmuteVideo":
        return unmuteVideo(payloads);
        break;
      case "unmuteAudio":
        return unmuteAudio(payloads);
        break;
      case "muteVideoRemote":
        return muteVideoRemote(payloads);
        break;
      case "muteAudioRemote":
        return muteAudioRemote(payloads);
        break;
      case "unmuteVideoRemote":
        return unmuteVideoRemote(payloads);
        break;
      case "unmuteAudioRemote":
        return unmuteAudioRemote(payloads);
        break;
      case "leave":
        return leave(payloads);
        break;
      default:
    }
  }

  static Map<String, dynamic> join(List<Map<String, dynamic>> payloads) {
    Map<String, dynamic> templete = {
      "app_id": null,
      // gateway use TLS-1.2 if "browser" container "Chrome" and verison > 74
      "browser": "Flutter Chrome/77.0.3865.35",
      "channel_key": null,
      "channel_name": null,
      "codec": null,
      "details": {},
      "extend": "",
      "has_changed_gateway": null,
      "mode": null,
      "process_id": null,
      "role": null,
      "sdk_version": VERSION,
      "session_id": null,
    };

    var message = _compose(templete, [payloads[0]]);
    var apResponse = payloads[1];
    apResponse.remove("addresses");
    message["ap_response"] = apResponse;

    return {
      "_type": "join",
      "_message": message,
    };
  }

  static Map<String, dynamic> publish(List<Map<String, dynamic>> payloads) {
    Map<String, dynamic> templete = {
      "video": null,
      "audio": null,
      "codec": null,
      "extend": "",
      "hq": false,
      "mode": null,
      "p2p_id": null,
      "sdp": "",
      "speech": false,
      "state": "offer",
      "stereo": false,
      "stream_type": "high",
    };
    var message = _compose(templete, payloads);
    final Map<String, dynamic> streamInfo =
        payloads[2] == null ? {} : payloads[2];
    final Map<String, dynamic> attributes = {
      "maxFrameRate": streamInfo["frameRate"],
      "maxVideoBW": streamInfo["bitRate"],
      "minFrameRate": streamInfo["frameRate"],
      "minVideoBW": streamInfo["bitRate"],
      "resolution": "${streamInfo["width"]}x${streamInfo["height"]}",
    };
    if (streamInfo["screen"] == true && streamInfo["video"] == false) {
      message["video"] = true;
    }
    message["attributes"] = attributes;

    return {
      "_type": "publish",
      "_message": message,
    };
  }

  static Map<String, dynamic> subscribe(List<Map<String, dynamic>> payloads) {
    Map<String, dynamic> templete = {
      "audio": null,
      "codec": null,
      "extend": "",
      "hq": false,
      "mode": null,
      "p2p_id": null,
      "sdp": "",
      "stream_id": null,
      "tcc": false,
      "video": null,
    };
    var message = _compose(templete, payloads);
    final Map<String, dynamic> streamInfo =
        payloads[2] == null ? {} : payloads[2];
    message["stream_id"] = streamInfo["streamId"];

    return {
      "_type": "subscribe",
      "_message": message,
    };
  }

  static Map<String, dynamic> unpublish(List<Map<String, dynamic>> payloads) {
    Map<String, dynamic> templete = {
      "stream_id": null,
      "stream_type": "high",
    };

    var message = _compose(templete, payloads);
    final Map<String, dynamic> streamInfo =
        payloads[0] == null ? {} : payloads[0];
    message["stream_id"] = streamInfo["streamId"];

    return {
      "_type": "unpublish",
      "_message": message,
    };
  }

  static Map<String, dynamic> unsubscribe(List<Map<String, dynamic>> payloads) {
    Map<String, dynamic> templete = {
      "stream_id": null,
    };

    var message = _compose(templete, payloads);
    final Map<String, dynamic> streamInfo =
        payloads[0] == null ? {} : payloads[0];
    message["stream_id"] = streamInfo["streamId"];

    return {
      "_type": "unsubscribe",
      "_message": message,
    };
  }

  static Map<String, dynamic> publishStats(
      List<Map<String, dynamic>> payloads) {
    Map<String, dynamic> templete = {
      "stream_type": "high",
      "stats": null,
    };
    return {
      "_type": "publish_stats",
      "_message": _compose(templete, payloads),
    };
  }

  static Map<String, dynamic> subscribeStats(
      List<Map<String, dynamic>> payloads) {
    Map<String, dynamic> templete = {
      "stream_id": "high",
      "stats": null,
    };

    var message = _compose(templete, [payloads[0]]);
    final Map<String, dynamic> streamInfo =
        payloads[1] == null ? {} : payloads[1];
    message["stream_id"] = streamInfo["streamId"];

    return {
      "_type": "subscribe_stats",
      "_message": message,
    };
  }

  static Map<String, dynamic> muteVideo(List<Map<String, dynamic>> payloads) {
    Map<String, dynamic> templete = {
      "action": "mute_local_video",
      "stream_id": null,
    };
    return {
      "_type": "control",
      "_message": _compose(templete, payloads),
    };
  }

  static Map<String, dynamic> muteAudio(List<Map<String, dynamic>> payloads) {
    Map<String, dynamic> templete = {
      "action": "mute_local_audio",
      "stream_id": null,
    };
    return {
      "_type": "control",
      "_message": _compose(templete, payloads),
    };
  }

  static Map<String, dynamic> unmuteVideo(List<Map<String, dynamic>> payloads) {
    Map<String, dynamic> templete = {
      "action": "unmute_local_video",
      "stream_id": null,
    };
    return {
      "_type": "control",
      "_message": _compose(templete, payloads),
    };
  }

  static Map<String, dynamic> unmuteAudio(List<Map<String, dynamic>> payloads) {
    Map<String, dynamic> templete = {
      "action": "unmute_local_audio",
      "stream_id": null,
    };
    return {
      "_type": "control",
      "_message": _compose(templete, payloads),
    };
  }

  static Map<String, dynamic> muteVideoRemote(
      List<Map<String, dynamic>> payloads) {
    Map<String, dynamic> templete = {
      "action": "mute_remote_video",
      "stream_id": null,
    };
    return {
      "_type": "control",
      "_message": _compose(templete, payloads),
    };
  }

  static Map<String, dynamic> muteAudioRemote(
      List<Map<String, dynamic>> payloads) {
    Map<String, dynamic> templete = {
      "action": "mute_remote_audio",
      "stream_id": null,
    };
    return {
      "_type": "control",
      "_message": _compose(templete, payloads),
    };
  }

  static Map<String, dynamic> unmuteVideoRemote(
      List<Map<String, dynamic>> payloads) {
    Map<String, dynamic> templete = {
      "action": "unmute_remote_video",
      "stream_id": null,
    };
    return {
      "_type": "control",
      "_message": _compose(templete, payloads),
    };
  }

  static Map<String, dynamic> unmuteAudioRemote(
      List<Map<String, dynamic>> payloads) {
    Map<String, dynamic> templete = {
      "action": "unmute_remote_audio",
      "stream_id": null,
    };
    return {
      "_type": "control",
      "_message": _compose(templete, payloads),
    };
  }

  static Map<String, dynamic> leave(List<Map<String, dynamic>> payloads) {
    final int uid = payloads[0]["uid"];
    return {
      "_id": uid,
      "_type": "leave",
      "_message": {},
    };
  }

  static Map<String, dynamic> _compose(
      Map<String, dynamic> templete, List<Map<String, dynamic>> payloads) {
    payloads.forEach((payload) {
      templete.addAll(payload);
    });
    return templete;
  }
}
