import 'dart:async';
import './signal.dart';
import '../../agora_stream.dart';
import './peerconnection.dart';

class PeerConnectionController {
  Signal signal;
  String clintId;
  Map<String, dynamic> sessionInfo;
  Map<int, PeerConnection> localConnections = {};
  Map<int, PeerConnection> remoteConnections = {};

  PeerConnectionController(this.signal, this.clintId, this.sessionInfo);

  Future<AgoraStream> publish(AgoraStream stream) async {
    var pc = await PeerConnection(signal, sessionInfo).init();
    localConnections[stream.streamId] = pc;
    return pc.publish(stream);
  }

  Future<AgoraStream> unpublish(AgoraStream stream) async {
    var pc = localConnections[stream.streamId];
    if (pc == null) return stream;
    await pc.unpublish(stream);
    localConnections.remove(stream.streamId);
    return stream;
  }

  Future<AgoraStream> subscribe(AgoraStream stream) async {
    var pc = await PeerConnection(signal, sessionInfo).init();
    remoteConnections[stream.streamId] = pc;
    return pc.subscribe(stream);
  }

  Future<AgoraStream> unsubscribe(AgoraStream stream) async {
    var pc = remoteConnections[stream.streamId];
    if (pc == null) return stream;
    await pc.unsubscribe(stream);
    remoteConnections.remove(stream.streamId);
    return stream;
  }

  Future<void> closePubConnection(int id) async {
    var con = localConnections[id];
    localConnections.remove(id);
    if (con != null) {
      await con.close();
    }
  }

  Future<void> closeSubConnection(int id) async {
    var con = remoteConnections[id];
    remoteConnections.remove(id);
    if (con != null) {
      var stream = con.stream;
      await con.close();
      stream.close();
    }
  }

  PeerConnection getPeerConnectionByP2PId(int id) {
    for(var remotePC in remoteConnections.values) {
      if(remotePC.p2pId == id) {
        return remotePC;
      }
    }
    for(var localPC in localConnections.values) {
      if(localPC.p2pId == id) {
        return localPC;
      }
    }
    return null;
  }

  clear() {
    localConnections.forEach((int id, PeerConnection pc) {
      pc?.close();
    });
    remoteConnections.forEach((int id, PeerConnection pc) {
      pc?.close();
    });
    localConnections = {};
    remoteConnections = {};
  }
}
