library events;

class EventEmitter {
  final Map _eventMap = {};

  on(String event, Function func) {
    _register(event, func, false);
  }

  once(String event, Function func) {
    _register(event, func, true);
  }

  emit(String event, Map<String, dynamic> data) {
    if (_eventMap[event] == null) {
      return;
    }
    List listeners = _eventMap[event];
    List gc = [];
    listeners.forEach((listener) {
      Function func = listener["target"];
      func(data);
      if (listener["once"]) {
        gc.add(listeners.indexOf(listener));
      }
    });
    gc.forEach((index) {
      listeners.removeAt(index);
    });
  }

  _register(String event, Function func, [bool once = false]) {
    if (_eventMap[event] == null) {
      _eventMap[event] = new List();
    }
    List listeners = _eventMap[event];
    listeners.add({"target": func, "once": once});
  }
}
