import 'dart:async';
import 'dart:convert';

import '../core/config.dart';
import './index.dart';
import 'package:http/http.dart' as http;

class Log {
  static const INFO = 1;
  static const WARN = 2;
  static const ERROR = 3;
  static const DEBUG = 4;
  static const SILENCE = 100;
  static int curlevel = 4;
  static int retryCount = 0;
  static int logId = 0;
  static bool isUpload = true;
  static bool isSending = false;
  static String cookieValue;

  static final _cache = [];
  static final http.Client httpClient = new http.Client();
  static final processId = Utils.getUUID();

  static Future<String> _sendLog(String logsText) async {
    try {
      var body = json.encode({
        "sdk_version": VERSION,
        "process_id": "processId-$processId",
        "payload": logsText
      });

      final url = "https://logservice-flutter.agora.io/upload/v1";

      Map<String, String> headers = {
        'Content-type': 'application/json',
        'Accept': 'application/json',
      };

      if(cookieValue != null) {
        headers['Cookie'] = cookieValue;
      }

      final response = await httpClient.post(url, body: body, headers: headers);

      final cookie = response?.headers["set-cookie"]?.split(";")[0];
      if(cookie != cookieValue) {
        cookieValue = cookie;
      }

      if (response.statusCode == 200) {
        if (response.body == "OK") {
          return Future.value(response.body);
        } else {
          return Future.error(response.body);
        }
      } else {
        return Future.error("Failed to log up, http: ");
      }
    } catch (err) {
      return Future.error(err);
    }
  }

  static sendLog(String logsText) async {
    isSending = true;
    await Future.delayed(new Duration(milliseconds: 3 * 1000));
    try {
      await _sendLog(logsText);
      retryCount = 0;
      var nextlogsText = _fetchLogs();
      if (nextlogsText == "0") {
        isSending = false;
        return;
      }
      await sendLog(nextlogsText);
    } catch (e) {
      print("upload log failed: $e");
      var delayTime = retryCount < 2 ? 200 : 10 * 1000;
      await Future.delayed(new Duration(microseconds: delayTime));
      sendLog(logsText);
    }
  }

  static String _fetchLogs() {
    var length = _cache.length;
    if (_cache.length == 0) {
      return "0";
    }
    var logs = _cache.sublist(0, length > 10 ? 10 : length);
    _cache.removeRange(0, length > 10 ? 10 : length);
    logs.replaceRange(
        0,
        logs.length,
        logs.map((log) {
          return {"log_item_id": logId++, "log_level": "", "payload_str": log};
        }).toList());
    var logText = jsonEncode(logs);
    return logText;
  }

  static _cacheLog(String log) {
    _cache.add(log);
    if (isSending == true) {
      return;
    }
    var logsText = _fetchLogs();
    sendLog(logsText);
  }

  static Map levelMap = {
    1: "INFO",
    2: "WARN",
    3: "ERROR",
    4: "DEBUG",
    100: "SILENCE"
  };

  static info(String log) {
    _print(INFO, log);
  }

  static warn(String log) {
    _print(WARN, log);
  }

  static error(String log) {
    _print(ERROR, log);
  }

  static debug(String log) {
    _print(DEBUG, log);
  }

  static silence(String log) {
    _print(SILENCE, log);
  }

  static _print(int level, String log) {
    var levelText = levelMap[level];
    var logText = "[$levelText]: $log";

    if (curlevel >= level) {
      print("Agora $logText");
    }

    if (isUpload) {
      var date = new DateTime.now().toString();
      _cacheLog("$date $logText");
    }
  }
}
