import 'dart:async';
import 'dart:convert';
import 'package:http/http.dart' as http;

const webcollectorURL = "https://webcollector-1.agora.io:6443/events/message";
const webcollectorURLbackup =
    "https://webcollector-2.agora.io:6443/events/message";

class EventReportManager {
  static final Map<String, EventReport> _sessionCollection = {};
  static final List _cache = [];

  static register({cid, sid, uid, cname}) {
    var reporter = new EventReport(cid: cid, sid: sid, uid: uid, cname: cname);
    _sessionCollection[sid] = reporter;
  }

  static report(String sid, String apiName, Map<String, dynamic> options) {
    if (sid == null) {
      if (_sessionCollection.isEmpty) {
        // TODO: cache
      }
      return;
    }

    var reporter = _sessionCollection[sid];
    if (reporter == null) {
      return;
    }

    if (apiName == "join") {
      reporter.uid = options["uid"];
      reporter.cid = options["cid"];
      reporter.cname = options["cname"];
    }

    reporter.report(apiName: apiName, options: options);
  }
}

class EventReport {
  int uid;
  int cid;
  int invokeId = 0;
  String sid;
  String cname;
  DateTime startTime;
  final http.Client httpClient = new http.Client();

  EventReport({this.cid, this.sid, this.uid, this.cname}) {
    startTime = DateTime.now();
  }

  report({
    apiName,
    options,
  }) async {
    final lts = DateTime.now();
    final elapse =
        (lts.microsecondsSinceEpoch - startTime.microsecondsSinceEpoch) ~/ 1000;
    final body = {
      "type": 'io.agora.pb.Wrtc.ApiInvoke',
      "data": {
        "cid": cid,
        "sid": sid,
        "uid": uid,
        "lts": lts.microsecondsSinceEpoch ~/ 1000,
        "cname": cname,
        "apiName": apiName,
        "elapse": elapse,
        "execElapse": 0,
        "invokeId": invokeId++,
        "options": json.encode(options),
        "success": true,
      }
    };

    try {
      await send(body, webcollectorURL);
    } catch (err) {
      send(body, webcollectorURLbackup);
    }
  }

  send(Map<String, dynamic> body, String url) async {
    Map<String, String> headers = {
      'Content-type': 'application/json',
    };

    var response =
        await httpClient.post(url, body: json.encode(body), headers: headers);

    if (response.statusCode == 200) {
      if (response.body == "OK") {
        return Future.value(response.body);
      }
    } else {
      return Future.error("Failed to report event");
    }
  }
}
